<?php

namespace TVn7\Migrations;

use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;

use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

use TVn7\DomainBundle\Entity\Rewrite;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version20161030212523 extends AbstractMigration implements ContainerAwareInterface
{
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    public function postUp(Schema $schema) {
        $em = $this->container->get('doctrine')->getManager();

        $this->write("Adding entries…");

        $rewrites = [
            "intranet" => "https://www.tvn7.fr/intranet",
            "tdb"      => "https://www.tvn7.fr/tdb",
            "admin"    => "https://www.tvn7.fr/tdb",
        ];

        foreach($rewrites as $subdomain=>$target) {
            $rewrite = new Rewrite();
            $rewrite->setSubdomain($subdomain)
                    ->setTarget($target);
            $em->persist($rewrite);

            $this->write(" Add '" . $subdomain . "' => '" . $target . "'");
        }

        $postesRepo = $em->getRepository('TVn7UserBundle:Poste');

        $this->write("Updating Postes");

        $role = "ROLE_DOMAIN_REWRITE_ADMIN";
        $postes = [ "respo-informatique" ];

        foreach($postes as $nameposte) {
            $poste = $postesRepo->findBy([ "name_canonical" => $nameposte]);
            if (count($poste) == 1) {
                $poste = $poste[0];
                $this->write(" Add '" . $role . "' to '" . $poste->getName() . "'");
                $poste->addRole($role);
                $membres = $poste->getMembres()->filter(function($membre) {
                    return ($membre->getAnnee() >= 2017);
                });
                foreach($membres as $membre) {
                    $this->write("  Add '" . $role . "' to '" . $membre . "' (" . $membre->getUsername() . ")");
                    $membre->addRole($role);
                }
            }
        }


        $em->flush();

    }
    
    /**
     * @param Schema $schema
     */
    public function up(Schema $schema)
    {
        // this up() migration is auto-generated, please modify it to your needs
        $this->abortIf($this->connection->getDatabasePlatform()->getName() != 'mysql', 'Migration can only be executed safely on \'mysql\'.');

        $this->addSql('CREATE TABLE tdb_domain_rewrite (id INT AUTO_INCREMENT NOT NULL, subdomain VARCHAR(255) NOT NULL, target VARCHAR(255) NOT NULL, count INT DEFAULT 0 NOT NULL, UNIQUE INDEX UNIQ_F7411E81C1D5962E (subdomain), PRIMARY KEY(id)) DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci ENGINE = InnoDB');
        $this->addSql('ALTER TABLE tdb_domain_rewrite ADD enabled TINYINT(1) DEFAULT \'1\' NOT NULL');
        $this->addSql('DROP FUNCTION IF EXISTS `rewrite_sel_inc`');
        $em = $this->container->get('doctrine')->getManager();
        $db = $em->getConnection()->getDatabase();
        $this->addSql('CREATE FUNCTION `rewrite_sel_inc`(`subd` VARCHAR(200)) RETURNS VARCHAR(200) NOT DETERMINISTIC CONTAINS SQL SQL SECURITY DEFINER begin declare ret VARCHAR(200); select target from `'.$db.'`.tdb_domain_rewrite where subdomain = subd and enabled = \'1\' into ret; select case when right(ret,1) = \'/\' then left(ret,length(ret)-1) else ret end into ret; UPDATE `'.$db.'`.tdb_domain_rewrite SET `count` = `count` + 1 where subdomain = subd and enabled = \'1\'; return ret; end');

    }

    /**
     * @param Schema $schema
     */
    public function down(Schema $schema)
    {
        // this down() migration is auto-generated, please modify it to your needs
        $this->abortIf($this->connection->getDatabasePlatform()->getName() != 'mysql', 'Migration can only be executed safely on \'mysql\'.');

        $this->addSql('DROP TABLE tdb_domain_rewrite');
        
    }
}
